
float3 view_position;
float3 light_position;
float4x4 view_proj_matrix;
float4x4 inv_world_matrix;
float4 Light_Ambient;
float4 Light1_Color;

texture texture0;
texture bumpmap;



sampler Texture0 = sampler_state
{
    Texture = <texture0>;
    mipfilter = LINEAR; 
};

sampler Bump= sampler_state
{
    Texture = <bumpmap>;
    mipfilter = LINEAR; 
};

struct VS_OUTPUT
{
   float4 Pos:       POSITION;
   float2 TexCoord:   TEXCOORD0;
   float4 LightDir:   TEXCOORD1;
   float3 HalfVect:   TEXCOORD2;
   float3 Normal:   TEXCOORD3;
   
};

//this is a generic lighting function using lambert and blinn-phong
vector Light_Point(float3 normal,float3 halfvec,vector light){
   // Specular
   float SpecularAttn =  max(0,pow(  dot(normal, halfvec),32));

   // Diffuse
   float AngleAttn = max(0, dot(normal, light) );
 
   // Compute final lighting
   vector color =  Light1_Color*(SpecularAttn+AngleAttn) +Light_Ambient;
   
   return color;
   
}

VS_OUTPUT vs_BumpandTextureMap(float4 inPos: POSITION, float3 inNormal: NORMAL,
                  float3 inTangent:TANGENT, float3 inBinormal:BINORMAL,
                  float2 inTxr: TEXCOORD0)
{
   VS_OUTPUT Out;

   // Compute the projected position and send out the texture coordinates
   Out.Pos = mul(inPos,view_proj_matrix);
   Out.TexCoord = inTxr;

   // Determine the distance from the light to the vertex and the direction
   float4 LightDir;
   LightDir.xyz = normalize(mul(light_position,inv_world_matrix) - inPos);



   // Determine the eye vector
   float3 EyeVector = normalize(mul(view_position,inv_world_matrix)-inPos);
   //calc half vector
   float3 HalfVector = normalize(LightDir.xyz+EyeVector);// average of the two vectors

   
   // Transform to texture space and output 
   // half vector and light direction
   float3x3 TangentSpace;
   TangentSpace[0] = normalize(inTangent);
   TangentSpace[1] = normalize(inBinormal);
   TangentSpace[2] = normalize(inNormal);
   TangentSpace=transpose(TangentSpace);
   
   //transform vectors to texture space
   Out.HalfVect = float4(mul(HalfVector.xyz,TangentSpace),1);
   Out.LightDir = float4(mul(LightDir.xyz,TangentSpace),1);

	Out.Normal=inNormal;
   return Out;
}



VS_OUTPUT vs_TextureOnly(float4 inPos: POSITION, float3 inNormal: NORMAL,
                  float3 inTangent:TANGENT, float3 inBinormal:BINORMAL,
                  float2 inTxr: TEXCOORD0)
{
   VS_OUTPUT Out;

   // Compute the projected position and send out the texture coordinates
   Out.Pos = mul(inPos,view_proj_matrix);
   Out.TexCoord = inTxr;

   // Determine the distance from the light to the vertex and the direction
   float4 LightDir;
   LightDir.xyz = normalize(mul(light_position,inv_world_matrix) - inPos);

   // Determine the eye vector
   float3 EyeVector = normalize(mul(view_position,inv_world_matrix)-inPos);

   
   Out.HalfVect = normalize(LightDir.xyz+EyeVector);
   Out.LightDir = float4(LightDir.xyz,1);
	Out.Normal=inNormal;
   return Out;
}







float4 ps_BumpandTextureMap(float2 inTxr:TEXCOORD0,float4 LightDir:TEXCOORD1,
               float3 HalfVect:TEXCOORD2) : COLOR 
{
   // Read normal map
   //texture values are stored in range 0 to 1, convert to range -1 to +1
   float3 normal = tex2D(Bump,inTxr) * 2 - 1; 

   // Output lighting color and texture map 
   return tex2D(Texture0,inTxr)*(Light_Point(normal,HalfVect,LightDir));
   
  
}

float4 ps_BumpOnly(float2 inTxr:TEXCOORD0,float4 LightDir:TEXCOORD1,
               float3 HalfVect:TEXCOORD2) : COLOR 
{
   // Read normal map
   //texture values are stored in range 0 to 1, convert to range -1 to +1
   float3 normal = tex2D(Bump,inTxr) * 2 - 1; 

   // Output lighting color and texture map 
   return (Light_Point(normal,HalfVect,LightDir));
   
  
}


float4 ps_TextureOnly(float2 inTxr:TEXCOORD0,float4 LightDir:TEXCOORD1,
               float3 HalfVect:TEXCOORD2,float3 normal:TEXCOORD3) : COLOR 
{

   // Output lighting color and texture map 
   return tex2D(Texture0,inTxr)*(Light_Point(normal,HalfVect,LightDir));
   
  
}

float4 ps_Neither(float2 inTxr:TEXCOORD0,float4 LightDir:TEXCOORD1,
               float3 HalfVect:TEXCOORD2,float3 normal:TEXCOORD3) : COLOR 
{
   // Output lighting color and texture map 
   return Light_Point(normal,HalfVect,LightDir);
   
  
}

technique BumpandTextureMap
{
    pass P0
    {
        
        vertexShader = compile vs_2_0 vs_BumpandTextureMap();
        pixelShader = compile ps_2_0 ps_BumpandTextureMap();
    }
}

technique BumpMapOnly
{
    pass P0
    {
        vertexShader = compile vs_2_0 vs_BumpandTextureMap();
        pixelShader = compile ps_2_0 ps_BumpOnly();
    }
}

technique TextureMapOnly
{
    pass P0
    {
        vertexShader = compile vs_2_0 vs_TextureOnly();
        pixelShader = compile ps_2_0 ps_TextureOnly();
    }
}

technique Neither
{
    pass P0
    {
        vertexShader = compile vs_2_0 vs_TextureOnly();
        pixelShader = compile ps_2_0 ps_Neither();
    }
}
