
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace XNALitTorus
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;        
        VertexPosTexNormalTanBitan[] toruspoints;
        Matrix view, proj, world;
        Effect effect;
        float angle = 0.0f;
        Texture2D texture,bump;
        Vector3 eye = new Vector3(0, 0, 7);
        Vector3 lightpos = new Vector3(0, 50, 0);
        bool bumpmapping = true, texturemapping = true, t_latch=true,b_latch=true;

        SpriteFont Font1;
        SpriteBatch spriteBatch;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            InitializeTransform();
            FillTourusPoints();
            InitializeTransform();
            effect = Content.Load<Effect>("bump");
            texture = Content.Load<Texture2D>("Fieldstone") as Texture2D;
            bump = Content.Load<Texture2D>("FieldstoneBumpDOT3") as Texture2D;

            effect.CurrentTechnique = effect.Techniques["BumpandTextureMap"];
            
            base.Initialize();
        }

        private void InitializeTransform()
        {

            view = Matrix.CreateLookAt(
                eye,
                Vector3.Zero,
                Vector3.Up
                );

            proj = Matrix.CreatePerspectiveFieldOfView(
                MathHelper.ToRadians(45),
                (float)graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height,
                1.0f, 100.0f);


        }

        private void FillTourusPoints()
        {

            int layers = 50;
            int slices = 50;
            float u, du = 1.0f / layers;
            float v, dv = 1.0f / slices;
            int point = 0;

            toruspoints = new VertexPosTexNormalTanBitan[layers * slices * 3 * 2];

            for (u = 0; u <= (1.0f-du); u += du)
            {
                for (v = 0; v <= (1.0f-dv); v += dv)
                {
                    //uppertriangle
                    SetVertex(ref toruspoints[point++],u,v);
                    SetVertex(ref toruspoints[point++], u + du, v);
                    SetVertex(ref toruspoints[point++], u, v + dv);

                    

                    //lowertriangle
                    SetVertex(ref toruspoints[point++], u+du, v);
                    SetVertex(ref toruspoints[point++], u + du, v+dv);
                    SetVertex(ref toruspoints[point++], u, v + dv);



                }

            }
        }

        private void SetVertex(ref VertexPosTexNormalTanBitan vertex, float u, float v)
        {
            // bitan & tan are calculated based on vectorts in direction of increasing u & v
            vertex.Position = XNALitTorus(u, v);
            Vector3 normal=XNATexturedTorusNormal(u, v);
            normal.Normalize();
            vertex.Normal = normal;
            vertex.Tex = new Vector2(u, v);
            Vector3 tan=XNALitTorus(u+0.1f, v) - XNALitTorus(u, v);
            tan.Normalize();
            vertex.Tan = tan;
            Vector3 bitan = XNALitTorus(u, v + 0.1f) - XNALitTorus(u, v);
            bitan.Normalize();
            vertex.Bitan = bitan;
            
        }

       

        private Vector3 XNATexturedTorusNormal(float u, float v)
        {
            u *= (float)Math.PI * 2.0f;
            v *= (float)Math.PI * 2.0f;


            return new Vector3(
                (float)Math.Cos(u) * ((float)Math.Cos(v)),
                (float)Math.Sin(v),
                (float)Math.Sin(u) * ((float)Math.Cos(v))
                );
        }

        private Vector3 XNALitTorus(float u, float v)
        {
            u *= (float)Math.PI * 2.0f;
            v *= (float)Math.PI * 2.0f;

            float R = 1.5f, r = 0.5f;
            return new Vector3(
                (float)Math.Cos(u) * (R + r * (float)Math.Cos(v)),
                r * (float)Math.Sin(v),
                (float)Math.Sin(u) * (R + r * (float)Math.Cos(v))
                );
        }
        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected

        override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            Font1 = Content.Load<SpriteFont>("SpriteFont1");

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadContent()
        {
           
                Content.Unload();
                
            
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            
            angle += 0.01f;
            // Allows the default game to exit on Xbox 360 and Windows
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            world = Matrix.CreateRotationX(angle) * Matrix.CreateRotationY(angle * 1.5f);
            base.Update(gameTime);


            if (Keyboard.GetState().IsKeyDown(Keys.T))
            {
                if (t_latch)
                    texturemapping = !texturemapping;
                t_latch = false;
            }
            else {
                t_latch = true;// use this to to prevent rapid togling while key held down
            }
            if (Keyboard.GetState().IsKeyDown(Keys.B))
            {
                if (b_latch)
                    bumpmapping = !bumpmapping;
                b_latch = false;
            }
            else
            {
                b_latch = true;
            }
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);
            graphics.GraphicsDevice.RenderState.CullMode = CullMode.CullCounterClockwiseFace;

            effect.Parameters["view_proj_matrix"].SetValue(world * view * proj);
            effect.Parameters["Light_Ambient"].SetValue(new Vector4(0.2f));
            effect.Parameters["Light1_Color"].SetValue(new Vector4(1.0f));
            effect.Parameters["inv_world_matrix"].SetValue(Matrix.Invert(world));
            effect.Parameters["texture0"].SetValue(texture); //diffuse texture
            effect.Parameters["bumpmap"].SetValue(bump); //normal map
            effect.Parameters["light_position"].SetValue(lightpos);
            effect.Parameters["view_position"].SetValue(eye);


            if (texturemapping && bumpmapping)
            {
                effect.CurrentTechnique = effect.Techniques["BumpandTextureMap"];
            }
 
            if (texturemapping && !bumpmapping)
            {
                effect.CurrentTechnique = effect.Techniques["TextureMapOnly"];
            }
            if (!texturemapping && bumpmapping)
            {
                effect.CurrentTechnique = effect.Techniques["BumpMapOnly"];
            }
            if (!texturemapping && !bumpmapping)
            {
                effect.CurrentTechnique = effect.Techniques["Neither"];
            }

            graphics.GraphicsDevice.VertexDeclaration = new VertexDeclaration(
                graphics.GraphicsDevice,
                VertexPosTexNormalTanBitan.VertexElements);

            effect.Begin();
            foreach (EffectPass pass in effect.CurrentTechnique.Passes)
            {
                pass.Begin();

                graphics.GraphicsDevice.DrawUserPrimitives<VertexPosTexNormalTanBitan>(
                PrimitiveType.TriangleList,
                toruspoints,
                0,
                toruspoints.Length / 3);
                pass.End();
            }
            effect.End();


            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Deferred, SaveStateMode.SaveState);

            // Draw Hello World
            string output = "press 't' and 'b' to toggle texture and bump mapping";

            // Find the center of the string
            Vector2 FontOrigin = Font1.MeasureString(output) / 2;
            // Draw the string
            spriteBatch.DrawString(Font1, output, FontOrigin, Color.LightGreen,
                0, FontOrigin, 1.0f, SpriteEffects.None, 0.5f);

            spriteBatch.End();

           
            
            base.Draw(gameTime);
        }
    }
}