#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace XNALitTorus
{
    
    [Serializable]
    public struct VertexPosTexNormalTanBitan
    {
        Vector3 pos;
        Vector2 tex;
        Vector3 normal, tan, bitan;

        public static readonly VertexElement[] VertexElements =
            new VertexElement[] { 
            new VertexElement(0,0,VertexElementFormat.Vector3,VertexElementMethod.Default,VertexElementUsage.Position,0),
            new VertexElement(0,sizeof(float)*3,VertexElementFormat.Vector2,VertexElementMethod.Default,VertexElementUsage.TextureCoordinate,0),
            new VertexElement(0,sizeof(float)*5,VertexElementFormat.Vector3,VertexElementMethod.Default,VertexElementUsage.Normal,0),
            new VertexElement(0,sizeof(float)*8,VertexElementFormat.Vector3,VertexElementMethod.Default,VertexElementUsage.Tangent,0),
            new VertexElement(0,sizeof(float)*11,VertexElementFormat.Vector3,VertexElementMethod.Default,VertexElementUsage.Binormal,0),
        };

        public VertexPosTexNormalTanBitan(Vector3 position, Vector2 uv, Vector3 normal, Vector3 tan, Vector3 bitan)
        {
            pos = position;
            tex = uv;
            this.normal = normal;
            this.tan = tan;
            this.bitan = bitan;
        }

        public static bool operator !=(VertexPosTexNormalTanBitan left, VertexPosTexNormalTanBitan right)
        {
            return left.GetHashCode() != right.GetHashCode();
        }

        public static bool operator ==(VertexPosTexNormalTanBitan left, VertexPosTexNormalTanBitan right)
        {
            return left.GetHashCode() == right.GetHashCode();
        }

        public override bool Equals(object obj)
        {
            return this == (VertexPosTexNormalTanBitan)obj;
        }

        public Vector3 Position { get { return pos; } set { pos = value; } }
        public Vector3 Normal { get { return normal; } set { normal = value; } }
        public Vector2 Tex { get { return tex; } set { tex = value; } }
        public Vector3 Tan { get { return tan; } set { tan = value; } }
        public Vector3 Bitan { get { return bitan; } set { bitan = value; } }

        public static int SizeInBytes { get { return sizeof(float) * 14; } }

        public override int GetHashCode()
        {
            return pos.GetHashCode() | tex.GetHashCode() | normal.GetHashCode() | tan.GetHashCode() | bitan.GetHashCode();
        }

        public override string ToString()
        {
            return string.Format("{0},{1},{2}", pos.X, pos.Y, pos.Z);
        }
    }
   
}
